@@ -145,10 +145,9 @@ impl Credentials {
     ///
     /// For example, given a name of `"pytorch"`, search for `UV_INDEX_PYTORCH_USERNAME` and
     /// `UV_INDEX_PYTORCH_PASSWORD`.
-    pub fn from_env(name: &str) -> Option<Self> {
-        let name = name.to_uppercase();
-        let username = std::env::var(EnvVars::index_username(&name)).ok();
-        let password = std::env::var(EnvVars::index_password(&name)).ok();
+    pub fn from_env(name: impl AsRef<str>) -> Option<Self> {
+        let username = std::env::var(EnvVars::index_username(name.as_ref())).ok();
+        let password = std::env::var(EnvVars::index_password(name.as_ref())).ok();
         if username.is_none() && password.is_none() {
             None
         } else {
