@@ -101,7 +101,7 @@ mod resolver {
     use uv_python::Interpreter;
     use uv_resolver::{
         FlatIndex, InMemoryIndex, Manifest, OptionsBuilder, PythonRequirement, RequiresPython,
-        ResolutionGraph, Resolver, ResolverMarkers,
+        ResolutionGraph, Resolver, ResolverEnvironment,
     };
     use uv_types::{BuildIsolation, EmptyInstalledPackages, HashStrategy, InFlight};
 
@@ -198,9 +198,9 @@ mod resolver {
         );
 
         let markers = if universal {
-            ResolverMarkers::universal(vec![])
+            ResolverEnvironment::universal(vec![])
         } else {
-            ResolverMarkers::specific_environment(ResolverMarkerEnvironment::from(MARKERS.clone()))
+            ResolverEnvironment::specific(ResolverMarkerEnvironment::from(MARKERS.clone()))
         };
 
         let resolver = Resolver::new(
