@@ -58,6 +58,7 @@ pub use crate::file::*;
 pub use crate::hash::*;
 pub use crate::id::*;
 pub use crate::index::*;
+pub use crate::index_name::*;
 pub use crate::index_url::*;
 pub use crate::installed::*;
 pub use crate::origin::*;
@@ -79,6 +80,7 @@ mod file;
 mod hash;
 mod id;
 mod index;
+mod index_name;
 mod index_url;
 mod installed;
 mod origin;
