@@ -32,6 +32,8 @@ pub enum Operator {
     /// `!= 1.2.*`
     NotEqualStar,
     /// `~=`
+    ///
+    /// Invariant: With `~=`, there are always at least 2 release segments.
     TildeEqual,
     /// `<`
     LessThan,
@@ -386,10 +388,10 @@ impl Version {
 
     /// Returns the local segments in this version, if any exist.
     #[inline]
-    pub fn local(&self) -> &[LocalSegment] {
+    pub fn local(&self) -> LocalVersionSlice {
         match *self.inner {
-            VersionInner::Small { ref small } => small.local(),
-            VersionInner::Full { ref full } => &full.local,
+            VersionInner::Small { ref small } => small.local_slice(),
+            VersionInner::Full { ref full } => full.local.as_slice(),
         }
     }
 
@@ -528,15 +530,33 @@ impl Version {
     /// Set the local segments and return the updated version.
     #[inline]
     #[must_use]
-    pub fn with_local(mut self, value: Vec<LocalSegment>) -> Self {
+    pub fn with_local_segments(mut self, value: Vec<LocalSegment>) -> Self {
         if value.is_empty() {
             self.without_local()
         } else {
-            self.make_full().local = value;
+            self.make_full().local = LocalVersion::Segments(value);
             self
         }
     }
 
+    /// Set the local version and return the updated version.
+    #[inline]
+    #[must_use]
+    pub fn with_local(mut self, value: LocalVersion) -> Self {
+        match value {
+            LocalVersion::Segments(segments) => self.with_local_segments(segments),
+            LocalVersion::Max => {
+                if let VersionInner::Small { ref mut small } = Arc::make_mut(&mut self.inner) {
+                    if small.set_local(LocalVersion::Max) {
+                        return self;
+                    }
+                }
+                self.make_full().local = value;
+                self
+            }
+        }
+    }
+
     /// For PEP 440 specifier matching: "Except where specifically noted below,
     /// local version identifiers MUST NOT be permitted in version specifiers,
     /// and local version labels MUST be ignored entirely when checking if
@@ -544,12 +564,12 @@ impl Version {
     #[inline]
     #[must_use]
     pub fn without_local(mut self) -> Self {
-        // A "small" version is already guaranteed not to have a local
-        // component, so we only need to do anything if we have a "full"
-        // version.
-        if let VersionInner::Full { ref mut full } = Arc::make_mut(&mut self.inner) {
-            full.local.clear();
+        if let VersionInner::Small { ref mut small } = Arc::make_mut(&mut self.inner) {
+            if small.set_local(LocalVersion::empty()) {
+                return self;
+            }
         }
+        self.make_full().local = LocalVersion::empty();
         self
     }
 
@@ -613,7 +633,7 @@ impl Version {
                 pre: small.pre(),
                 post: small.post(),
                 dev: small.dev(),
-                local: vec![],
+                local: small.local(),
             };
             *self = Self {
                 inner: Arc::new(VersionInner::Full { full }),
@@ -710,14 +730,12 @@ impl std::fmt::Display for Version {
         let local = if self.local().is_empty() {
             String::new()
         } else {
-            format!(
-                "+{}",
-                self.local()
-                    .iter()
-                    .map(ToString::to_string)
-                    .collect::<Vec<String>>()
-                    .join(".")
-            )
+            match self.local() {
+                LocalVersionSlice::Segments(_) => {
+                    format!("+{}", self.local())
+                }
+                LocalVersionSlice::Max => "+".to_string(),
+            }
         };
         write!(f, "{epoch}{release}{pre}{post}{dev}{local}")
     }
@@ -833,16 +851,16 @@ impl FromStr for Version {
 /// * Bytes 5, 4 and 3 correspond to the second, third and fourth release
 ///   segments, respectively.
 /// * Bytes 2, 1 and 0 represent *one* of the following:
-///   `min, .devN, aN, bN, rcN, <no suffix>, .postN, max`.
+///   `min, .devN, aN, bN, rcN, <no suffix>, local, .postN, max`.
 ///   Its representation is thus:
-///   * The most significant 3 bits of Byte 2 corresponds to a value in
-///     the range 0-6 inclusive, corresponding to min, dev, pre-a, pre-b, pre-rc,
-///     no-suffix or post releases, respectively. `min` is a special version that
-///     does not exist in PEP 440, but is used here to represent the smallest
-///     possible version, preceding any `dev`, `pre`, `post` or releases. `max` is
-///     an analogous concept for the largest possible version, following any `post`
-///     or local releases.
-///   * The low 5 bits combined with the bits in bytes 1 and 0 correspond
+///   * The most significant 4 bits of Byte 2 corresponds to a value in
+///     the range 0-8 inclusive, corresponding to min, dev, pre-a, pre-b,
+///     pre-rc, no-suffix, post or max releases, respectively. `min` is a
+///     special version that does not exist in PEP 440, but is used here to
+///     represent the smallest possible version, preceding any `dev`, `pre`,
+///     `post` or releases. `max` is an analogous concept for the largest
+///     possible version, following any `post` or local releases.
+///   * The low 4 bits combined with the bits in bytes 1 and 0 correspond
 ///     to the release number of the suffix, if one exists. If there is no
 ///     suffix, then these bits are always 0.
 ///
@@ -900,20 +918,52 @@ struct VersionSmall {
 }
 
 impl VersionSmall {
+    // Constants for each suffix kind. They form an enumeration.
+    //
+    // The specific values are assigned in a way that provides the suffix kinds
+    // their ordering. i.e., No suffix should sort after a dev suffix but
+    // before a post suffix.
+    //
+    // The maximum possible suffix value is SUFFIX_KIND_MASK. If you need to
+    // add another suffix value and you're at the max, then the mask must gain
+    // another bit. And adding another bit to the mask will require taking it
+    // from somewhere else. (Usually the suffix version.)
+    //
+    // NOTE: If you do change the bit format here, you'll need to bump any
+    // cache versions in uv that use rkyv with `Version` in them. That includes
+    // *at least* the "simple" cache.
     const SUFFIX_MIN: u64 = 0;
     const SUFFIX_DEV: u64 = 1;
     const SUFFIX_PRE_ALPHA: u64 = 2;
     const SUFFIX_PRE_BETA: u64 = 3;
     const SUFFIX_PRE_RC: u64 = 4;
     const SUFFIX_NONE: u64 = 5;
-    const SUFFIX_POST: u64 = 6;
-    const SUFFIX_MAX: u64 = 7;
-    const SUFFIX_MAX_VERSION: u64 = 0x001F_FFFF;
+    const SUFFIX_LOCAL: u64 = 6;
+    const SUFFIX_POST: u64 = 7;
+    const SUFFIX_MAX: u64 = 8;
+
+    // The mask to get only the release segment bits.
+    //
+    // NOTE: If you change the release mask to have more or less bits,
+    // then you'll also need to change `push_release` below and also
+    // `Parser::parse_fast`.
+    const SUFFIX_RELEASE_MASK: u64 = 0xFFFF_FFFF_FF00_0000;
+    // The mask to get the version suffix.
+    const SUFFIX_VERSION_MASK: u64 = 0x000F_FFFF;
+    // The number of bits used by the version suffix. Shifting the `repr`
+    // right by this number of bits should put the suffix kind in the least
+    // significant bits.
+    const SUFFIX_VERSION_BIT_LEN: u64 = 20;
+    // The mask to get only the suffix kind, after shifting right by the
+    // version bits. If you need to add a bit here, then you'll probably need
+    // to take a bit from the suffix version. (Which requires a change to both
+    // the mask and the bit length above.)
+    const SUFFIX_KIND_MASK: u64 = 0b1111;
 
     #[inline]
     fn new() -> Self {
         Self {
-            repr: 0x0000_0000_00A0_0000,
+            repr: Self::SUFFIX_NONE << Self::SUFFIX_VERSION_BIT_LEN,
             release: [0, 0, 0, 0],
             len: 0,
         }
@@ -941,7 +991,7 @@ impl VersionSmall {
 
     #[inline]
     fn clear_release(&mut self) {
-        self.repr &= !0xFFFF_FFFF_FF00_0000;
+        self.repr &= !Self::SUFFIX_RELEASE_MASK;
         self.release = [0, 0, 0, 0];
         self.len = 0;
     }
@@ -982,11 +1032,8 @@ impl VersionSmall {
 
     #[inline]
     fn set_post(&mut self, value: Option<u64>) -> bool {
-        if self.min().is_some()
-            || self.pre().is_some()
-            || self.dev().is_some()
-            || self.max().is_some()
-        {
+        let suffix_kind = self.suffix_kind();
+        if !(suffix_kind == Self::SUFFIX_NONE || suffix_kind == Self::SUFFIX_POST) {
             return value.is_none();
         }
         match value {
@@ -994,7 +1041,7 @@ impl VersionSmall {
                 self.set_suffix_kind(Self::SUFFIX_NONE);
             }
             Some(number) => {
-                if number > Self::SUFFIX_MAX_VERSION {
+                if number > Self::SUFFIX_VERSION_MASK {
                     return false;
                 }
                 self.set_suffix_kind(Self::SUFFIX_POST);
@@ -1029,10 +1076,11 @@ impl VersionSmall {
 
     #[inline]
     fn set_pre(&mut self, value: Option<Prerelease>) -> bool {
-        if self.min().is_some()
-            || self.dev().is_some()
-            || self.post().is_some()
-            || self.max().is_some()
+        let suffix_kind = self.suffix_kind();
+        if !(suffix_kind == Self::SUFFIX_NONE
+            || suffix_kind == Self::SUFFIX_PRE_ALPHA
+            || suffix_kind == Self::SUFFIX_PRE_BETA
+            || suffix_kind == Self::SUFFIX_PRE_RC)
         {
             return value.is_none();
         }
@@ -1041,7 +1089,7 @@ impl VersionSmall {
                 self.set_suffix_kind(Self::SUFFIX_NONE);
             }
             Some(Prerelease { kind, number }) => {
-                if number > Self::SUFFIX_MAX_VERSION {
+                if number > Self::SUFFIX_VERSION_MASK {
                     return false;
                 }
                 match kind {
@@ -1072,11 +1120,8 @@ impl VersionSmall {
 
     #[inline]
     fn set_dev(&mut self, value: Option<u64>) -> bool {
-        if self.min().is_some()
-            || self.pre().is_some()
-            || self.post().is_some()
-            || self.max().is_some()
-        {
+        let suffix_kind = self.suffix_kind();
+        if !(suffix_kind == Self::SUFFIX_NONE || suffix_kind == Self::SUFFIX_DEV) {
             return value.is_none();
         }
         match value {
@@ -1084,7 +1129,7 @@ impl VersionSmall {
                 self.set_suffix_kind(Self::SUFFIX_NONE);
             }
             Some(number) => {
-                if number > Self::SUFFIX_MAX_VERSION {
+                if number > Self::SUFFIX_VERSION_MASK {
                     return false;
                 }
                 self.set_suffix_kind(Self::SUFFIX_DEV);
@@ -1105,11 +1150,8 @@ impl VersionSmall {
 
     #[inline]
     fn set_min(&mut self, value: Option<u64>) -> bool {
-        if self.dev().is_some()
-            || self.pre().is_some()
-            || self.post().is_some()
-            || self.max().is_some()
-        {
+        let suffix_kind = self.suffix_kind();
+        if !(suffix_kind == Self::SUFFIX_NONE || suffix_kind == Self::SUFFIX_MIN) {
             return value.is_none();
         }
         match value {
@@ -1117,7 +1159,7 @@ impl VersionSmall {
                 self.set_suffix_kind(Self::SUFFIX_NONE);
             }
             Some(number) => {
-                if number > Self::SUFFIX_MAX_VERSION {
+                if number > Self::SUFFIX_VERSION_MASK {
                     return false;
                 }
                 self.set_suffix_kind(Self::SUFFIX_MIN);
@@ -1138,11 +1180,8 @@ impl VersionSmall {
 
     #[inline]
     fn set_max(&mut self, value: Option<u64>) -> bool {
-        if self.dev().is_some()
-            || self.pre().is_some()
-            || self.post().is_some()
-            || self.min().is_some()
-        {
+        let suffix_kind = self.suffix_kind();
+        if !(suffix_kind == Self::SUFFIX_NONE || suffix_kind == Self::SUFFIX_MAX) {
             return value.is_none();
         }
         match value {
@@ -1150,7 +1189,7 @@ impl VersionSmall {
                 self.set_suffix_kind(Self::SUFFIX_NONE);
             }
             Some(number) => {
-                if number > Self::SUFFIX_MAX_VERSION {
+                if number > Self::SUFFIX_VERSION_MASK {
                     return false;
                 }
                 self.set_suffix_kind(Self::SUFFIX_MAX);
@@ -1161,16 +1200,45 @@ impl VersionSmall {
     }
 
     #[inline]
-    #[allow(clippy::unused_self)]
-    fn local(&self) -> &[LocalSegment] {
-        // A "small" version is never used if the version has a non-zero number
-        // of local segments.
-        &[]
+    fn local(&self) -> LocalVersion {
+        if self.suffix_kind() == Self::SUFFIX_LOCAL {
+            LocalVersion::Max
+        } else {
+            LocalVersion::empty()
+        }
+    }
+
+    #[inline]
+    fn local_slice(&self) -> LocalVersionSlice {
+        if self.suffix_kind() == Self::SUFFIX_LOCAL {
+            LocalVersionSlice::Max
+        } else {
+            LocalVersionSlice::empty()
+        }
+    }
+
+    #[inline]
+    fn set_local(&mut self, value: LocalVersion) -> bool {
+        let suffix_kind = self.suffix_kind();
+        if !(suffix_kind == Self::SUFFIX_NONE || suffix_kind == Self::SUFFIX_LOCAL) {
+            return value.is_empty();
+        }
+        match value {
+            LocalVersion::Max => {
+                self.set_suffix_kind(Self::SUFFIX_LOCAL);
+                true
+            }
+            LocalVersion::Segments(segments) if segments.is_empty() => {
+                self.set_suffix_kind(Self::SUFFIX_NONE);
+                true
+            }
+            LocalVersion::Segments(_) => false,
+        }
     }
 
     #[inline]
     fn suffix_kind(&self) -> u64 {
-        let kind = (self.repr >> 21) & 0b111;
+        let kind = (self.repr >> Self::SUFFIX_VERSION_BIT_LEN) & Self::SUFFIX_KIND_MASK;
         debug_assert!(kind <= Self::SUFFIX_MAX);
         kind
     }
@@ -1178,22 +1246,22 @@ impl VersionSmall {
     #[inline]
     fn set_suffix_kind(&mut self, kind: u64) {
         debug_assert!(kind <= Self::SUFFIX_MAX);
-        self.repr &= !0x00E0_0000;
-        self.repr |= kind << 21;
-        if kind == Self::SUFFIX_NONE {
+        self.repr &= !(Self::SUFFIX_KIND_MASK << Self::SUFFIX_VERSION_BIT_LEN);
+        self.repr |= kind << Self::SUFFIX_VERSION_BIT_LEN;
+        if kind == Self::SUFFIX_NONE || kind == Self::SUFFIX_LOCAL {
             self.set_suffix_version(0);
         }
     }
 
     #[inline]
     fn suffix_version(&self) -> u64 {
-        self.repr & 0x001F_FFFF
+        self.repr & Self::SUFFIX_VERSION_MASK
     }
 
     #[inline]
     fn set_suffix_version(&mut self, value: u64) {
-        debug_assert!(value <= 0x001F_FFFF);
-        self.repr &= !0x001F_FFFF;
+        debug_assert!(value <= Self::SUFFIX_VERSION_MASK);
+        self.repr &= !Self::SUFFIX_VERSION_MASK;
         self.repr |= value;
     }
 }
@@ -1250,7 +1318,7 @@ struct VersionFull {
     ///
     /// Local versions allow multiple segments separated by periods, such as `deadbeef.1.2.3`, see
     /// [`LocalSegment`] for details on the semantics.
-    local: Vec<LocalSegment>,
+    local: LocalVersion,
     /// An internal-only segment that does not exist in PEP 440, used to
     /// represent the smallest possible version of a release, preceding any
     /// `dev`, `pre`, `post` or releases.
@@ -1381,6 +1449,111 @@ impl std::fmt::Display for Prerelease {
     }
 }
 
+/// Either a sequence of local segments or [`LocalVersion::Sentinel`], an internal-only value that
+/// compares greater than all other local versions.
+#[derive(Eq, PartialEq, Debug, Clone, Hash)]
+#[cfg_attr(
+    feature = "rkyv",
+    derive(rkyv::Archive, rkyv::Deserialize, rkyv::Serialize)
+)]
+#[cfg_attr(feature = "rkyv", rkyv(derive(Debug, Eq, PartialEq, PartialOrd, Ord)))]
+pub enum LocalVersion {
+    /// A sequence of local segments.
+    Segments(Vec<LocalSegment>),
+    /// An internal-only value that compares greater to all other local versions.
+    Max,
+}
+
+/// Like [`LocalVersion`], but using a slice
+#[derive(Eq, PartialEq, Debug, Clone, Hash)]
+pub enum LocalVersionSlice<'a> {
+    /// Like [`LocalVersion::Segments`]
+    Segments(&'a [LocalSegment]),
+    /// Like [`LocalVersion::Sentinel`]
+    Max,
+}
+
+impl LocalVersion {
+    /// Return an empty local version.
+    pub fn empty() -> Self {
+        Self::Segments(Vec::new())
+    }
+
+    /// Returns `true` if the local version is empty.
+    pub fn is_empty(&self) -> bool {
+        match self {
+            Self::Segments(segments) => segments.is_empty(),
+            Self::Max => false,
+        }
+    }
+
+    /// Convert the local version segments into a slice.
+    pub fn as_slice(&self) -> LocalVersionSlice<'_> {
+        match self {
+            LocalVersion::Segments(segments) => LocalVersionSlice::Segments(segments),
+            LocalVersion::Max => LocalVersionSlice::Max,
+        }
+    }
+
+    /// Clear the local version segments, if they exist.
+    pub fn clear(&mut self) {
+        match self {
+            Self::Segments(segments) => segments.clear(),
+            Self::Max => *self = Self::Segments(Vec::new()),
+        }
+    }
+}
+
+/// Output the local version identifier string.
+///
+/// [`LocalVersionSlice::Max`] maps to `"[max]"` which is otherwise an illegal local
+/// version because `[` and `]` are not allowed.
+impl std::fmt::Display for LocalVersionSlice<'_> {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            LocalVersionSlice::Segments(segments) => {
+                for (i, segment) in segments.iter().enumerate() {
+                    if i > 0 {
+                        write!(f, ".")?;
+                    }
+                    write!(f, "{segment}")?;
+                }
+                Ok(())
+            }
+            LocalVersionSlice::Max => write!(f, "[max]"),
+        }
+    }
+}
+
+impl PartialOrd for LocalVersionSlice<'_> {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl Ord for LocalVersionSlice<'_> {
+    fn cmp(&self, other: &Self) -> Ordering {
+        match (self, other) {
+            (LocalVersionSlice::Segments(lv1), LocalVersionSlice::Segments(lv2)) => lv1.cmp(lv2),
+            (LocalVersionSlice::Segments(_), LocalVersionSlice::Max) => Ordering::Less,
+            (LocalVersionSlice::Max, LocalVersionSlice::Segments(_)) => Ordering::Greater,
+            (LocalVersionSlice::Max, LocalVersionSlice::Max) => Ordering::Equal,
+        }
+    }
+}
+
+impl LocalVersionSlice<'_> {
+    /// Return an empty local version.
+    pub const fn empty() -> Self {
+        Self::Segments(&[])
+    }
+
+    /// Returns `true` if the local version is empty.
+    pub fn is_empty(&self) -> bool {
+        matches!(self, Self::Segments(&[]))
+    }
+}
+
 /// A part of the [local version identifier](<https://peps.python.org/pep-0440/#local-version-identifiers>)
 ///
 /// Local versions are a mess:
@@ -1575,17 +1748,11 @@ impl<'a> Parser<'a> {
         *release.get_mut(usize::from(len))? = cur;
         len += 1;
         let small = VersionSmall {
-            // Clippy warns about no-ops like `(0x00 << 16)`, but I
-            // think it makes the bit logic much clearer, and makes it
-            // explicit that nothing was forgotten.
-            #[allow(clippy::identity_op)]
             repr: (u64::from(release[0]) << 48)
                 | (u64::from(release[1]) << 40)
                 | (u64::from(release[2]) << 32)
                 | (u64::from(release[3]) << 24)
-                | (0xA0 << 16)
-                | (0x00 << 8)
-                | (0x00 << 0),
+                | (VersionSmall::SUFFIX_NONE << VersionSmall::SUFFIX_VERSION_BIT_LEN),
             release: [
                 u64::from(release[0]),
                 u64::from(release[1]),
@@ -1828,7 +1995,7 @@ impl<'a> Parser<'a> {
             .with_pre(self.pre)
             .with_post(self.post)
             .with_dev(self.dev)
-            .with_local(self.local);
+            .with_local(LocalVersion::Segments(self.local));
         VersionPattern {
             version,
             wildcard: self.wildcard,
@@ -2299,7 +2466,7 @@ pub(crate) fn compare_release(this: &[u64], other: &[u64]) -> Ordering {
 /// implementation
 ///
 /// [pep440-suffix-ordering]: https://peps.python.org/pep-0440/#summary-of-permitted-suffixes-and-relative-ordering
-fn sortable_tuple(version: &Version) -> (u64, u64, Option<u64>, u64, &[LocalSegment]) {
+fn sortable_tuple(version: &Version) -> (u64, u64, Option<u64>, u64, LocalVersionSlice) {
     // If the version is a "max" version, use a post version larger than any possible post version.
     let post = if version.max().is_some() {
         Some(u64::MAX)
