@@ -2,7 +2,7 @@ use rustc_hash::FxHashMap;
 use uv_distribution_types::IndexUrl;
 use uv_normalize::PackageName;
 
-use crate::resolver::ResolverMarkers;
+use crate::resolver::ResolverEnvironment;
 use crate::ResolveError;
 
 /// See [`crate::resolver::ForkState`].
@@ -20,27 +20,17 @@ impl ForkIndexes {
         &mut self,
         package_name: &PackageName,
         index: &IndexUrl,
-        fork_markers: &ResolverMarkers,
+        env: &ResolverEnvironment,
     ) -> Result<(), ResolveError> {
         if let Some(previous) = self.0.insert(package_name.clone(), index.clone()) {
             if &previous != index {
                 let mut conflicts = vec![previous.to_string(), index.to_string()];
                 conflicts.sort();
-                return match fork_markers {
-                    ResolverMarkers::Universal { .. } | ResolverMarkers::SpecificEnvironment(_) => {
-                        Err(ResolveError::ConflictingIndexesUniversal(
-                            package_name.clone(),
-                            conflicts,
-                        ))
-                    }
-                    ResolverMarkers::Fork(fork_markers) => {
-                        Err(ResolveError::ConflictingIndexesFork {
-                            package_name: package_name.clone(),
-                            indexes: conflicts,
-                            fork_markers: fork_markers.clone(),
-                        })
-                    }
-                };
+                return Err(ResolveError::ConflictingIndexesForEnvironment {
+                    package_name: package_name.clone(),
+                    indexes: conflicts,
+                    env: env.clone(),
+                });
             }
         }
         Ok(())
