@@ -14,92 +14,105 @@ use uv_configuration::{DevGroupsManifest, EditableMode, ExtrasSpecification, Ins
 use uv_distribution_filename::{DistExtension, SourceDistExtension};
 use uv_fs::Simplified;
 use uv_git::GitReference;
-use uv_normalize::{ExtraName, PackageName};
+use uv_normalize::ExtraName;
 use uv_pep508::MarkerTree;
 use uv_pypi_types::{ParsedArchiveUrl, ParsedGitUrl};
 
 use crate::graph_ops::marker_reachability;
 use crate::lock::{Package, PackageId, Source};
-use crate::{Lock, LockError};
+use crate::{InstallTarget, LockError};
 
-type LockGraph<'lock> = Graph<&'lock Package, Edge, Directed>;
-
-#[derive(Debug, Clone, PartialEq, Eq)]
-struct Node<'lock> {
-    package: &'lock Package,
-    marker: MarkerTree,
-}
+type LockGraph<'lock> = Graph<Node<'lock>, Edge, Directed>;
 
 /// An export of a [`Lock`] that renders in `requirements.txt` format.
 #[derive(Debug)]
 pub struct RequirementsTxtExport<'lock> {
-    nodes: Vec<Node<'lock>>,
+    nodes: Vec<Requirement<'lock>>,
     hashes: bool,
     editable: EditableMode,
 }
 
 impl<'lock> RequirementsTxtExport<'lock> {
     pub fn from_lock(
-        lock: &'lock Lock,
-        root_name: &PackageName,
+        target: InstallTarget<'lock>,
         extras: &ExtrasSpecification,
         dev: &DevGroupsManifest,
         editable: EditableMode,
         hashes: bool,
         install_options: &'lock InstallOptions,
     ) -> Result<Self, LockError> {
-        let size_guess = lock.packages.len();
+        let size_guess = target.lock().packages.len();
         let mut petgraph = LockGraph::with_capacity(size_guess, size_guess);
         let mut inverse = FxHashMap::with_capacity_and_hasher(size_guess, FxBuildHasher);
 
         let mut queue: VecDeque<(&Package, Option<&ExtraName>)> = VecDeque::new();
         let mut seen = FxHashSet::default();
 
+        let root = petgraph.add_node(Node::Root);
+
         // Add the workspace package to the queue.
-        let root = lock
-            .find_by_name(root_name)
-            .expect("found too many packages matching root")
-            .expect("could not find root");
+        for root_name in target.packages() {
+            let dist = target
+                .lock()
+                .find_by_name(root_name)
+                .expect("found too many packages matching root")
+                .expect("could not find root");
 
-        if dev.prod() {
-            // Add the base package.
-            queue.push_back((root, None));
-
-            // Add any extras.
-            match extras {
-                ExtrasSpecification::None => {}
-                ExtrasSpecification::All => {
-                    for extra in root.optional_dependencies.keys() {
-                        queue.push_back((root, Some(extra)));
-                    }
+            if dev.prod() {
+                // Add the workspace package to the graph.
+                if let Entry::Vacant(entry) = inverse.entry(&dist.id) {
+                    entry.insert(petgraph.add_node(Node::Package(dist)));
                 }
-                ExtrasSpecification::Some(extras) => {
-                    for extra in extras {
-                        queue.push_back((root, Some(extra)));
+
+                // Add an edge from the root.
+                let index = inverse[&dist.id];
+                petgraph.add_edge(root, index, MarkerTree::TRUE);
+
+                // Push its dependencies on the queue.
+                queue.push_back((dist, None));
+                match extras {
+                    ExtrasSpecification::None => {}
+                    ExtrasSpecification::All => {
+                        for extra in dist.optional_dependencies.keys() {
+                            queue.push_back((dist, Some(extra)));
+                        }
+                    }
+                    ExtrasSpecification::Some(extras) => {
+                        for extra in extras {
+                            queue.push_back((dist, Some(extra)));
+                        }
                     }
                 }
             }
 
-            // Add the root package to the graph.
-            inverse.insert(&root.id, petgraph.add_node(root));
-        }
+            // Add any development dependencies.
+            for group in dev.iter() {
+                for dep in dist.dependency_groups.get(group).into_iter().flatten() {
+                    let dep_dist = target.lock().find_by_id(&dep.package_id);
 
-        // Add any dev dependencies.
-        for group in dev.iter() {
-            for dep in root.dependency_groups.get(group).into_iter().flatten() {
-                let dep_dist = lock.find_by_id(&dep.package_id);
+                    // Add the dependency to the graph.
+                    if let Entry::Vacant(entry) = inverse.entry(&dep.package_id) {
+                        entry.insert(petgraph.add_node(Node::Package(dep_dist)));
+                    }
 
-                // Add the dependency to the graph.
-                if let Entry::Vacant(entry) = inverse.entry(&dep.package_id) {
-                    entry.insert(petgraph.add_node(dep_dist));
-                }
+                    // Add an edge from the root. Development dependencies may be installed without
+                    // installing the workspace package itself (which can never have markers on it
+                    // anyway), so they're directly connected to the root.
+                    let dep_index = inverse[&dep.package_id];
+                    petgraph.add_edge(
+                        root,
+                        dep_index,
+                        dep.simplified_marker.as_simplified_marker_tree().clone(),
+                    );
 
-                if seen.insert((&dep.package_id, None)) {
-                    queue.push_back((dep_dist, None));
-                }
-                for extra in &dep.extra {
-                    if seen.insert((&dep.package_id, Some(extra))) {
-                        queue.push_back((dep_dist, Some(extra)));
+                    // Push its dependencies on the queue.
+                    if seen.insert((&dep.package_id, None)) {
+                        queue.push_back((dep_dist, None));
+                    }
+                    for extra in &dep.extra {
+                        if seen.insert((&dep.package_id, Some(extra))) {
+                            queue.push_back((dep_dist, Some(extra)));
+                        }
                     }
                 }
             }
@@ -122,11 +135,11 @@ impl<'lock> RequirementsTxtExport<'lock> {
             };
 
             for dep in deps {
-                let dep_dist = lock.find_by_id(&dep.package_id);
+                let dep_dist = target.lock().find_by_id(&dep.package_id);
 
                 // Add the dependency to the graph.
                 if let Entry::Vacant(entry) = inverse.entry(&dep.package_id) {
-                    entry.insert(petgraph.add_node(dep_dist));
+                    entry.insert(petgraph.add_node(Node::Package(dep_dist)));
                 }
 
                 // Add the edge.
@@ -152,12 +165,20 @@ impl<'lock> RequirementsTxtExport<'lock> {
         let mut reachability = marker_reachability(&petgraph, &[]);
 
         // Collect all packages.
-        let mut nodes: Vec<Node> = petgraph
+        let mut nodes = petgraph
             .node_references()
-            .filter(|(_index, package)| {
-                install_options.include_package(&package.id.name, Some(root_name), lock.members())
+            .filter_map(|(index, node)| match node {
+                Node::Root => None,
+                Node::Package(package) => Some((index, package)),
             })
-            .map(|(index, package)| Node {
+            .filter(|(_index, package)| {
+                install_options.include_package(
+                    &package.id.name,
+                    target.project_name(),
+                    target.lock().members(),
+                )
+            })
+            .map(|(index, package)| Requirement {
                 package,
                 marker: reachability.remove(&index).unwrap_or_default(),
             })
@@ -165,7 +186,7 @@ impl<'lock> RequirementsTxtExport<'lock> {
 
         // Sort the nodes, such that unnamed URLs (editables) appear at the top.
         nodes.sort_unstable_by(|a, b| {
-            NodeComparator::from(a.package).cmp(&NodeComparator::from(b.package))
+            RequirementComparator::from(a.package).cmp(&RequirementComparator::from(b.package))
         });
 
         Ok(Self {
@@ -179,7 +200,7 @@ impl<'lock> RequirementsTxtExport<'lock> {
 impl std::fmt::Display for RequirementsTxtExport<'_> {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         // Write out each package.
-        for Node { package, marker } in &self.nodes {
+        for Requirement { package, marker } in &self.nodes {
             match &package.id.source {
                 Source::Registry(_) => {
                     write!(f, "{}=={}", package.id.name, package.id.version)?;
@@ -261,17 +282,31 @@ impl std::fmt::Display for RequirementsTxtExport<'_> {
     }
 }
 
+/// A node in the [`LockGraph`].
+#[derive(Debug, Clone, PartialEq, Eq)]
+enum Node<'lock> {
+    Root,
+    Package(&'lock Package),
+}
+
 /// The edges of the [`LockGraph`].
 type Edge = MarkerTree;
 
+/// A flat requirement, with its associated marker.
+#[derive(Debug, Clone, PartialEq, Eq)]
+struct Requirement<'lock> {
+    package: &'lock Package,
+    marker: MarkerTree,
+}
+
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
-enum NodeComparator<'lock> {
+enum RequirementComparator<'lock> {
     Editable(&'lock Path),
     Path(&'lock Path),
     Package(&'lock PackageId),
 }
 
-impl<'lock> From<&'lock Package> for NodeComparator<'lock> {
+impl<'lock> From<&'lock Package> for RequirementComparator<'lock> {
     fn from(value: &'lock Package) -> Self {
         match &value.id.source {
             Source::Path(path) | Source::Directory(path) => Self::Path(path),
