@@ -2,7 +2,7 @@ use std::env;
 use std::ops::Deref;
 use std::path::{Path, PathBuf};
 
-use tracing::debug;
+use etcetera::BaseStrategy;
 
 use uv_fs::Simplified;
 use uv_static::EnvVars;
@@ -42,16 +42,17 @@ impl FilesystemOptions {
         let root = dir.join("uv");
         let file = root.join("uv.toml");
 
-        debug!("Searching for user configuration in: `{}`", file.display());
+        tracing::debug!("Searching for user configuration in: `{}`", file.display());
         match read_file(&file) {
             Ok(options) => {
-                debug!("Found user configuration in: `{}`", file.display());
+                tracing::debug!("Found user configuration in: `{}`", file.display());
+                validate_uv_toml(&file, &options)?;
                 Ok(Some(Self(options)))
             }
             Err(Error::Io(err)) if err.kind() == std::io::ErrorKind::NotFound => Ok(None),
             Err(_) if !dir.is_dir() => {
                 // Ex) `XDG_CONFIG_HOME=/dev/null`
-                debug!(
+                tracing::debug!(
                     "User configuration directory `{}` does not exist or is not a directory",
                     dir.display()
                 );
@@ -65,7 +66,7 @@ impl FilesystemOptions {
         let Some(file) = system_config_file() else {
             return Ok(None);
         };
-        debug!("Found system configuration in: `{}`", file.display());
+        tracing::debug!("Found system configuration in: `{}`", file.display());
         Ok(Some(Self(read_file(&file)?)))
     }
 
@@ -82,11 +83,11 @@ impl FilesystemOptions {
                 Ok(None) => {
                     // Continue traversing the directory tree.
                 }
-                Err(Error::PyprojectToml(file, err)) => {
+                Err(Error::PyprojectToml(path, err)) => {
                     // If we see an invalid `pyproject.toml`, warn but continue.
                     warn_user!(
                         "Failed to parse `{}` during settings discovery:\n{}",
-                        file.cyan(),
+                        path.user_display().cyan(),
                         textwrap::indent(&err.to_string(), "  ")
                     );
                 }
@@ -107,7 +108,7 @@ impl FilesystemOptions {
         match fs_err::read_to_string(&path) {
             Ok(content) => {
                 let options: Options = toml::from_str(&content)
-                    .map_err(|err| Error::UvToml(path.user_display().to_string(), err))?;
+                    .map_err(|err| Error::UvToml(path.clone(), Box::new(err)))?;
 
                 // If the directory also contains a `[tool.uv]` table in a `pyproject.toml` file,
                 // warn.
@@ -123,7 +124,8 @@ impl FilesystemOptions {
                     }
                 }
 
-                debug!("Found workspace configuration at `{}`", path.display());
+                tracing::debug!("Found workspace configuration at `{}`", path.display());
+                validate_uv_toml(&path, &options)?;
                 return Ok(Some(Self(options)));
             }
             Err(err) if err.kind() == std::io::ErrorKind::NotFound => {}
@@ -136,23 +138,23 @@ impl FilesystemOptions {
             Ok(content) => {
                 // Parse, but skip any `pyproject.toml` that doesn't have a `[tool.uv]` section.
                 let pyproject: PyProjectToml = toml::from_str(&content)
-                    .map_err(|err| Error::PyprojectToml(path.user_display().to_string(), err))?;
+                    .map_err(|err| Error::PyprojectToml(path.clone(), Box::new(err)))?;
                 let Some(tool) = pyproject.tool else {
-                    debug!(
+                    tracing::debug!(
                         "Skipping `pyproject.toml` in `{}` (no `[tool]` section)",
                         dir.display()
                     );
                     return Ok(None);
                 };
                 let Some(options) = tool.uv else {
-                    debug!(
+                    tracing::debug!(
                         "Skipping `pyproject.toml` in `{}` (no `[tool.uv]` section)",
                         dir.display()
                     );
                     return Ok(None);
                 };
 
-                debug!("Found workspace configuration at `{}`", path.display());
+                tracing::debug!("Found workspace configuration at `{}`", path.display());
                 return Ok(Some(Self(options)));
             }
             Err(err) if err.kind() == std::io::ErrorKind::NotFound => {}
@@ -176,23 +178,12 @@ impl From<Options> for FilesystemOptions {
 
 /// Returns the path to the user configuration directory.
 ///
-/// This is similar to the `config_dir()` returned by the `dirs` crate, but it uses the
-/// `XDG_CONFIG_HOME` environment variable on both Linux _and_ macOS, rather than the
-/// `Application Support` directory on macOS.
+/// On Windows, use, e.g., C:\Users\Alice\AppData\Roaming
+/// On Linux and macOS, use `XDG_CONFIG_HOME` or $HOME/.config, e.g., /home/alice/.config.
 fn user_config_dir() -> Option<PathBuf> {
-    // On Windows, use, e.g., `C:\Users\Alice\AppData\Roaming`.
-    #[cfg(windows)]
-    {
-        dirs_sys::known_folder_roaming_app_data()
-    }
-
-    // On Linux and macOS, use, e.g., `/home/alice/.config`.
-    #[cfg(not(windows))]
-    {
-        env::var_os(EnvVars::XDG_CONFIG_HOME)
-            .and_then(dirs_sys::is_absolute_path)
-            .or_else(|| dirs_sys::home_dir().map(|path| path.join(".config")))
-    }
+    etcetera::choose_base_strategy()
+        .map(|dirs| dirs.config_dir())
+        .ok()
 }
 
 #[cfg(not(windows))]
@@ -241,28 +232,70 @@ fn system_config_file() -> Option<PathBuf> {
         // Fallback to `/etc/uv/uv.toml` if `XDG_CONFIG_DIRS` is not set or no valid
         // path is found.
         let candidate = Path::new("/etc/uv/uv.toml");
-        candidate.is_file().then(|| candidate.to_path_buf())
+        match candidate.try_exists() {
+            Ok(true) => Some(candidate.to_path_buf()),
+            Ok(false) => None,
+            Err(err) => {
+                tracing::warn!("Failed to query system configuration file: {err}");
+                None
+            }
+        }
     }
 }
 
 /// Load [`Options`] from a `uv.toml` file.
 fn read_file(path: &Path) -> Result<Options, Error> {
     let content = fs_err::read_to_string(path)?;
-    let options: Options = toml::from_str(&content)
-        .map_err(|err| Error::UvToml(path.user_display().to_string(), err))?;
+    let options: Options =
+        toml::from_str(&content).map_err(|err| Error::UvToml(path.to_path_buf(), Box::new(err)))?;
     Ok(options)
 }
 
+/// Validate that an [`Options`] schema is compatible with `uv.toml`.
+fn validate_uv_toml(path: &Path, options: &Options) -> Result<(), Error> {
+    // The `uv.toml` format is not allowed to include any of the following, which are
+    // permitted by the schema since they _can_ be included in `pyproject.toml` files
+    // (and we want to use `deny_unknown_fields`).
+    if options.workspace.is_some() {
+        return Err(Error::PyprojectOnlyField(path.to_path_buf(), "workspace"));
+    }
+    if options.sources.is_some() {
+        return Err(Error::PyprojectOnlyField(path.to_path_buf(), "sources"));
+    }
+    if options.dev_dependencies.is_some() {
+        return Err(Error::PyprojectOnlyField(
+            path.to_path_buf(),
+            "dev-dependencies",
+        ));
+    }
+    if options.default_groups.is_some() {
+        return Err(Error::PyprojectOnlyField(
+            path.to_path_buf(),
+            "default-groups",
+        ));
+    }
+    if options.managed.is_some() {
+        return Err(Error::PyprojectOnlyField(path.to_path_buf(), "managed"));
+    }
+    if options.package.is_some() {
+        return Err(Error::PyprojectOnlyField(path.to_path_buf(), "package"));
+    }
+    Ok(())
+}
+
 #[derive(thiserror::Error, Debug)]
 pub enum Error {
     #[error(transparent)]
     Io(#[from] std::io::Error),
 
-    #[error("Failed to parse: `{0}`")]
-    PyprojectToml(String, #[source] toml::de::Error),
+    #[error("Failed to parse: `{}`", _0.user_display())]
+    PyprojectToml(PathBuf, #[source] Box<toml::de::Error>),
 
-    #[error("Failed to parse: `{0}`")]
-    UvToml(String, #[source] toml::de::Error),
+    #[error("Failed to parse: `{}`", _0.user_display())]
+    UvToml(PathBuf, #[source] Box<toml::de::Error>),
+
+    #[error("Failed to parse: `{}`. The `{1}` field is not allowed in a `uv.toml` file. `{1}` is only applicable in the context of a project, and should be placed in a `pyproject.toml` file instead.", _0.user_display())]
+    PyprojectOnlyField(PathBuf, &'static str),
 }
 
 #[cfg(test)]
