@@ -98,7 +98,7 @@ impl CachedEnvironment {
         }
 
         // Create the environment in the cache, then relocate it to its content-addressed location.
-        let temp_dir = cache.environment()?;
+        let temp_dir = cache.venv_dir()?;
         let venv = uv_virtualenv::create_venv(
             temp_dir.path(),
             interpreter,
