@@ -9,14 +9,14 @@ use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
     Concurrency, Constraints, DevGroupsSpecification, ExtrasSpecification, GroupsSpecification,
-    LowerBound, Reinstall, Upgrade,
+    LowerBound, Reinstall, TrustedHost, Upgrade,
 };
 use uv_dispatch::BuildDispatch;
 use uv_distribution::DistributionDatabase;
 use uv_distribution_types::{
     Index, Resolution, UnresolvedRequirement, UnresolvedRequirementSpecification,
 };
-use uv_fs::Simplified;
+use uv_fs::{Simplified, CWD};
 use uv_git::ResolvedRepositoryReference;
 use uv_installer::{SatisfiesResult, SitePackages};
 use uv_normalize::{GroupName, PackageName, DEV_DEPENDENCIES};
@@ -26,7 +26,7 @@ use uv_pypi_types::Requirement;
 use uv_python::{
     EnvironmentPreference, Interpreter, InvalidEnvironmentKind, PythonDownloads, PythonEnvironment,
     PythonInstallation, PythonPreference, PythonRequest, PythonVariant, PythonVersionFile,
-    VersionRequest,
+    VersionFileDiscoveryOptions, VersionRequest,
 };
 use uv_requirements::upgrade::{read_lock_requirements, LockedRequirements};
 use uv_requirements::{NamedRequirementsResolver, RequirementsSpecification};
@@ -34,6 +34,7 @@ use uv_resolver::{
     FlatIndex, Lock, OptionsBuilder, PythonRequirement, RequiresPython, ResolutionGraph,
     ResolverEnvironment,
 };
+use uv_scripts::Pep723Item;
 use uv_types::{BuildIsolation, EmptyInstalledPackages, HashStrategy};
 use uv_warnings::{warn_user, warn_user_once};
 use uv_workspace::dependency_groups::DependencyGroupError;
@@ -89,13 +90,13 @@ pub(crate) enum ProjectError {
     RequiresPythonProjectIncompatibility(Version, RequiresPython),
 
     #[error("The requested interpreter resolved to Python {0}, which is incompatible with the script's Python requirement: `{1}`")]
-    RequestedPythonScriptIncompatibility(Version, VersionSpecifiers),
+    RequestedPythonScriptIncompatibility(Version, RequiresPython),
 
     #[error("The Python request from `{0}` resolved to Python {1}, which is incompatible with the script's Python requirement: `{2}`")]
-    DotPythonVersionScriptIncompatibility(String, Version, VersionSpecifiers),
+    DotPythonVersionScriptIncompatibility(String, Version, RequiresPython),
 
     #[error("The resolved Python interpreter (Python {0}) is incompatible with the script's Python requirement: `{1}`")]
-    RequiresPythonScriptIncompatibility(Version, VersionSpecifiers),
+    RequiresPythonScriptIncompatibility(Version, RequiresPython),
 
     #[error("The requested interpreter resolved to Python {0}, which is incompatible with the project's Python requirement: `{1}`. However, a workspace member (`{member}`) supports Python {3}. To install the workspace member on its own, navigate to `{path}`, then run `{venv}` followed by `{install}`.", member = _2.cyan(), venv = format!("uv venv --python {_0}").green(), install = "uv pip install -e .".green(), path = _4.user_display().cyan() )]
     RequestedMemberIncompatibility(
@@ -222,7 +223,7 @@ pub(crate) fn find_requires_python(workspace: &Workspace) -> Option<RequiresPyth
 #[allow(clippy::result_large_err)]
 pub(crate) fn validate_requires_python(
     interpreter: &Interpreter,
-    workspace: &Workspace,
+    workspace: Option<&Workspace>,
     requires_python: &RequiresPython,
     source: &PythonRequestSource,
 ) -> Result<(), ProjectError> {
@@ -235,7 +236,7 @@ pub(crate) fn validate_requires_python(
     // a library in the workspace is compatible with Python >=3.8, the user may attempt
     // to sync on Python 3.8. This will fail, but we should provide a more helpful error
     // message.
-    for (name, member) in workspace.packages() {
+    for (name, member) in workspace.into_iter().flat_map(Workspace::packages) {
         let Some(project) = member.pyproject_toml().project.as_ref() else {
             continue;
         };
@@ -255,7 +256,7 @@ pub(crate) fn validate_requires_python(
                 }
                 PythonRequestSource::DotPythonVersion(file) => {
                     Err(ProjectError::DotPythonVersionMemberIncompatibility(
-                        file.to_string(),
+                        file.path().user_display().to_string(),
                         interpreter.python_version().clone(),
                         requires_python.clone(),
                         name.clone(),
@@ -285,7 +286,7 @@ pub(crate) fn validate_requires_python(
         }
         PythonRequestSource::DotPythonVersion(file) => {
             Err(ProjectError::DotPythonVersionProjectIncompatibility(
-                file.to_string(),
+                file.path().user_display().to_string(),
                 interpreter.python_version().clone(),
                 requires_python.clone(),
             ))
@@ -299,6 +300,49 @@ pub(crate) fn validate_requires_python(
     }
 }
 
+/// Returns an error if the [`Interpreter`] does not satisfy script or workspace `requires-python`.
+#[allow(clippy::result_large_err)]
+pub(crate) fn validate_script_requires_python(
+    interpreter: &Interpreter,
+    workspace: Option<&Workspace>,
+    requires_python: &RequiresPython,
+    requires_python_source: &RequiresPythonSource,
+    request_source: &PythonRequestSource,
+) -> Result<(), ProjectError> {
+    match requires_python_source {
+        RequiresPythonSource::Project => {
+            validate_requires_python(interpreter, workspace, requires_python, request_source)?;
+        }
+        RequiresPythonSource::Script => {}
+    };
+
+    if requires_python.contains(interpreter.python_version()) {
+        return Ok(());
+    }
+
+    match request_source {
+        PythonRequestSource::UserRequest => {
+            Err(ProjectError::RequestedPythonScriptIncompatibility(
+                interpreter.python_version().clone(),
+                requires_python.clone(),
+            ))
+        }
+        PythonRequestSource::DotPythonVersion(file) => {
+            Err(ProjectError::DotPythonVersionScriptIncompatibility(
+                file.file_name().to_string(),
+                interpreter.python_version().clone(),
+                requires_python.clone(),
+            ))
+        }
+        PythonRequestSource::RequiresPython => {
+            Err(ProjectError::RequiresPythonScriptIncompatibility(
+                interpreter.python_version().clone(),
+                requires_python.clone(),
+            ))
+        }
+    }
+}
+
 /// An interpreter suitable for the project.
 #[derive(Debug)]
 #[allow(clippy::large_enum_variant)]
@@ -314,47 +358,68 @@ pub(crate) enum PythonRequestSource {
     /// The request was provided by the user.
     UserRequest,
     /// The request was inferred from a `.python-version` or `.python-versions` file.
-    DotPythonVersion(String),
+    DotPythonVersion(PythonVersionFile),
     /// The request was inferred from a `pyproject.toml` file.
     RequiresPython,
 }
 
+impl std::fmt::Display for PythonRequestSource {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            PythonRequestSource::UserRequest => write!(f, "explicit request"),
+            PythonRequestSource::DotPythonVersion(file) => {
+                write!(f, "version file at `{}`", file.path().user_display())
+            }
+            PythonRequestSource::RequiresPython => write!(f, "`requires-python` metadata"),
+        }
+    }
+}
+
 /// The resolved Python request and requirement for a [`Workspace`].
 #[derive(Debug, Clone)]
 pub(crate) struct WorkspacePython {
     /// The source of the Python request.
-    source: PythonRequestSource,
+    pub(crate) source: PythonRequestSource,
     /// The resolved Python request, computed by considering (1) any explicit request from the user
     /// via `--python`, (2) any implicit request from the user via `.python-version`, and (3) any
     /// `Requires-Python` specifier in the `pyproject.toml`.
-    python_request: Option<PythonRequest>,
+    pub(crate) python_request: Option<PythonRequest>,
     /// The resolved Python requirement for the project, computed by taking the intersection of all
     /// `Requires-Python` specifiers in the workspace.
-    requires_python: Option<RequiresPython>,
+    pub(crate) requires_python: Option<RequiresPython>,
 }
 
 impl WorkspacePython {
     /// Determine the [`WorkspacePython`] for the current [`Workspace`].
     pub(crate) async fn from_request(
         python_request: Option<PythonRequest>,
-        workspace: &Workspace,
+        workspace: Option<&Workspace>,
+        project_dir: &Path,
+        no_config: bool,
     ) -> Result<Self, ProjectError> {
-        let requires_python = find_requires_python(workspace);
+        let requires_python = workspace.and_then(find_requires_python);
+
+        let workspace_root = workspace.map(Workspace::install_path);
 
         let (source, python_request) = if let Some(request) = python_request {
             // (1) Explicit request from user
             let source = PythonRequestSource::UserRequest;
             let request = Some(request);
             (source, request)
-        } else if let Some(file) =
-            PythonVersionFile::discover(workspace.install_path(), false, false).await?
+        } else if let Some(file) = PythonVersionFile::discover(
+            project_dir,
+            &VersionFileDiscoveryOptions::default()
+                .with_stop_discovery_at(workspace_root.map(PathBuf::as_ref))
+                .with_no_config(no_config),
+        )
+        .await?
         {
             // (2) Request from `.python-version`
-            let source = PythonRequestSource::DotPythonVersion(file.file_name().to_string());
+            let source = PythonRequestSource::DotPythonVersion(file.clone());
             let request = file.into_version();
             (source, request)
         } else {
-            // (3) `Requires-Python` in `pyproject.toml`
+            // (3) `requires-python` in `pyproject.toml`
             let request = requires_python
                 .as_ref()
                 .map(RequiresPython::specifiers)
@@ -368,6 +433,87 @@ impl WorkspacePython {
             (source, request)
         };
 
+        if let Some(python_request) = python_request.as_ref() {
+            debug!(
+                "Using Python request `{}` from {source}",
+                python_request.to_canonical_string()
+            );
+        };
+
+        Ok(Self {
+            source,
+            python_request,
+            requires_python,
+        })
+    }
+}
+
+/// The source of a `Requires-Python` specifier.
+#[derive(Debug, Clone)]
+pub(crate) enum RequiresPythonSource {
+    /// From the PEP 723 inline script metadata.
+    Script,
+    /// From a `pyproject.toml` in a workspace.
+    Project,
+}
+
+/// The resolved Python request and requirement for a [`Pep723Script`]
+#[derive(Debug, Clone)]
+pub(crate) struct ScriptPython {
+    /// The source of the Python request.
+    pub(crate) source: PythonRequestSource,
+    /// The resolved Python request, computed by considering (1) any explicit request from the user
+    /// via `--python`, (2) any implicit request from the user via `.python-version`, (3) any
+    /// `Requires-Python` specifier in the script metadata, and (4) any `Requires-Python` specifier
+    /// in the `pyproject.toml`.
+    pub(crate) python_request: Option<PythonRequest>,
+    /// The resolved Python requirement for the script and its source.
+    pub(crate) requires_python: Option<(RequiresPython, RequiresPythonSource)>,
+}
+
+impl ScriptPython {
+    /// Determine the [`ScriptPython`] for the current [`Workspace`].
+    pub(crate) async fn from_request(
+        python_request: Option<PythonRequest>,
+        workspace: Option<&Workspace>,
+        script: &Pep723Item,
+        no_config: bool,
+    ) -> Result<Self, ProjectError> {
+        // First, discover a requirement from the workspace
+        let WorkspacePython {
+            mut source,
+            mut python_request,
+            requires_python,
+        } = WorkspacePython::from_request(
+            python_request,
+            workspace,
+            script.path().and_then(Path::parent).unwrap_or(&**CWD),
+            no_config,
+        )
+        .await?;
+
+        // If the script has a `requires-python` specifier, prefer that over one from the workspace.
+        let requires_python =
+            if let Some(requires_python_specifiers) = script.metadata().requires_python.as_ref() {
+                if python_request.is_none() {
+                    python_request = Some(PythonRequest::Version(VersionRequest::Range(
+                        requires_python_specifiers.clone(),
+                        PythonVariant::Default,
+                    )));
+                    source = PythonRequestSource::RequiresPython;
+                }
+                Some((
+                    RequiresPython::from_specifiers(requires_python_specifiers),
+                    RequiresPythonSource::Script,
+                ))
+            } else {
+                requires_python.map(|requirement| (requirement, RequiresPythonSource::Project))
+            };
+
+        if let Some(python_request) = python_request.as_ref() {
+            debug!("Using Python request {python_request} from {source}");
+        };
+
         Ok(Self {
             source,
             python_request,
@@ -380,11 +526,14 @@ impl ProjectInterpreter {
     /// Discover the interpreter to use in the current [`Workspace`].
     pub(crate) async fn discover(
         workspace: &Workspace,
+        project_dir: &Path,
         python_request: Option<PythonRequest>,
         python_preference: PythonPreference,
         python_downloads: PythonDownloads,
         connectivity: Connectivity,
         native_tls: bool,
+        allow_insecure_host: &[TrustedHost],
+        no_config: bool,
         cache: &Cache,
         printer: Printer,
     ) -> Result<Self, ProjectError> {
@@ -393,7 +542,8 @@ impl ProjectInterpreter {
             source,
             python_request,
             requires_python,
-        } = WorkspacePython::from_request(python_request, workspace).await?;
+        } = WorkspacePython::from_request(python_request, Some(workspace), project_dir, no_config)
+            .await?;
 
         // Read from the virtual environment first.
         let venv = workspace.venv();
@@ -401,11 +551,15 @@ impl ProjectInterpreter {
             Ok(venv) => {
                 if python_request.as_ref().map_or(true, |request| {
                     if request.satisfied(venv.interpreter(), cache) {
-                        debug!("The virtual environment's Python version satisfies `{request}`");
+                        debug!(
+                            "The virtual environment's Python version satisfies `{}`",
+                            request.to_canonical_string()
+                        );
                         true
                     } else {
                         debug!(
-                            "The virtual environment's Python version does not satisfy `{request}`"
+                            "The virtual environment's Python version does not satisfy `{}`",
+                            request.to_canonical_string()
                         );
                         false
                     }
@@ -459,7 +613,8 @@ impl ProjectInterpreter {
 
         let client_builder = BaseClientBuilder::default()
             .connectivity(connectivity)
-            .native_tls(native_tls);
+            .native_tls(native_tls)
+            .allow_insecure_host(allow_insecure_host.to_vec());
 
         let reporter = PythonDownloadReporter::single(printer);
 
@@ -497,7 +652,7 @@ impl ProjectInterpreter {
         }
 
         if let Some(requires_python) = requires_python.as_ref() {
-            validate_requires_python(&interpreter, workspace, requires_python, &source)?;
+            validate_requires_python(&interpreter, Some(workspace), requires_python, &source)?;
         }
 
         Ok(Self::Interpreter(interpreter))
@@ -520,16 +675,21 @@ pub(crate) async fn get_or_init_environment(
     python_downloads: PythonDownloads,
     connectivity: Connectivity,
     native_tls: bool,
+    allow_insecure_host: &[TrustedHost],
+    no_config: bool,
     cache: &Cache,
     printer: Printer,
 ) -> Result<PythonEnvironment, ProjectError> {
     match ProjectInterpreter::discover(
         workspace,
+        workspace.install_path().as_ref(),
         python,
         python_preference,
         python_downloads,
         connectivity,
         native_tls,
+        allow_insecure_host,
+        no_config,
         cache,
         printer,
     )
@@ -626,6 +786,7 @@ pub(crate) async fn resolve_names(
     connectivity: Connectivity,
     concurrency: Concurrency,
     native_tls: bool,
+    allow_insecure_host: &[TrustedHost],
     cache: &Cache,
     printer: Printer,
 ) -> Result<Vec<Requirement>, uv_requirements::Error> {
@@ -650,7 +811,6 @@ pub(crate) async fn resolve_names(
         index_locations,
         index_strategy,
         keyring_provider,
-        allow_insecure_host,
         resolution: _,
         prerelease: _,
         dependency_metadata,
@@ -680,7 +840,7 @@ pub(crate) async fn resolve_names(
         .index_urls(index_locations.index_urls())
         .index_strategy(*index_strategy)
         .keyring(*keyring_provider)
-        .allow_insecure_host(allow_insecure_host.clone())
+        .allow_insecure_host(allow_insecure_host.to_vec())
         .markers(interpreter.markers())
         .platform(interpreter.platform())
         .build();
@@ -778,6 +938,7 @@ pub(crate) async fn resolve_environment<'a>(
     connectivity: Connectivity,
     concurrency: Concurrency,
     native_tls: bool,
+    allow_insecure_host: &[TrustedHost],
     cache: &Cache,
     printer: Printer,
 ) -> Result<ResolutionGraph, ProjectError> {
@@ -787,7 +948,6 @@ pub(crate) async fn resolve_environment<'a>(
         index_locations,
         index_strategy,
         keyring_provider,
-        allow_insecure_host,
         resolution,
         prerelease,
         dependency_metadata,
@@ -953,6 +1113,7 @@ pub(crate) async fn sync_environment(
     connectivity: Connectivity,
     concurrency: Concurrency,
     native_tls: bool,
+    allow_insecure_host: &[TrustedHost],
     cache: &Cache,
     printer: Printer,
 ) -> anyhow::Result<PythonEnvironment> {
@@ -960,7 +1121,6 @@ pub(crate) async fn sync_environment(
         index_locations,
         index_strategy,
         keyring_provider,
-        allow_insecure_host,
         dependency_metadata,
         config_setting,
         no_build_isolation,
@@ -1106,6 +1266,7 @@ pub(crate) async fn update_environment(
     connectivity: Connectivity,
     concurrency: Concurrency,
     native_tls: bool,
+    allow_insecure_host: &[TrustedHost],
     cache: &Cache,
     printer: Printer,
 ) -> anyhow::Result<EnvironmentUpdate> {
@@ -1115,7 +1276,6 @@ pub(crate) async fn update_environment(
         index_locations,
         index_strategy,
         keyring_provider,
-        allow_insecure_host,
         resolution,
         prerelease,
         dependency_metadata,
@@ -1186,7 +1346,7 @@ pub(crate) async fn update_environment(
         .index_urls(index_locations.index_urls())
         .index_strategy(*index_strategy)
         .keyring(*keyring_provider)
-        .allow_insecure_host(allow_insecure_host.clone())
+        .allow_insecure_host(allow_insecure_host.to_vec())
         .markers(interpreter.markers())
         .platform(interpreter.platform())
         .build();
@@ -1322,13 +1482,14 @@ pub(crate) async fn update_environment(
     })
 }
 
-/// Determine the [`RequiresPython`] requirement for a PEP 723 script.
-pub(crate) async fn script_python_requirement(
+/// Determine the [`RequiresPython`] requirement for a new PEP 723 script.
+pub(crate) async fn init_script_python_requirement(
     python: Option<&str>,
     directory: &Path,
     no_pin_python: bool,
     python_preference: PythonPreference,
     python_downloads: PythonDownloads,
+    no_config: bool,
     client_builder: &BaseClientBuilder<'_>,
     cache: &Cache,
     reporter: &PythonDownloadReporter,
@@ -1338,9 +1499,12 @@ pub(crate) async fn script_python_requirement(
         PythonRequest::parse(request)
     } else if let (false, Some(request)) = (
         no_pin_python,
-        PythonVersionFile::discover(directory, false, false)
-            .await?
-            .and_then(PythonVersionFile::into_version),
+        PythonVersionFile::discover(
+            directory,
+            &VersionFileDiscoveryOptions::default().with_no_config(no_config),
+        )
+        .await?
+        .and_then(PythonVersionFile::into_version),
     ) {
         // (2) Request from `.python-version`
         request
