@@ -47,22 +47,33 @@ fn help() {
               Disable automatic downloads of Python. [env: "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet                      Do not print any output
-      -v, --verbose...                 Use verbose output
-          --color <COLOR_CHOICE>       Control colors in output [default: auto] [possible values: auto,
-                                       always, never]
-          --native-tls                 Whether to load TLS certificates from the platform's native
-                                       certificate store [env: UV_NATIVE_TLS=]
-          --offline                    Disable network access
-          --no-progress                Hide all progress outputs [env: UV_NO_PROGRESS=]
-          --directory <DIRECTORY>      Change to the given directory prior to running the command
-          --project <PROJECT>          Run the command within the given project directory
-          --config-file <CONFIG_FILE>  The path to a `uv.toml` file to use for configuration [env:
-                                       UV_CONFIG_FILE=]
-          --no-config                  Avoid discovering configuration files (`pyproject.toml`,
-                                       `uv.toml`) [env: UV_NO_CONFIG=]
-      -h, --help                       Display the concise help for this command
-      -V, --version                    Display the uv version
+      -q, --quiet
+              Do not print any output
+      -v, --verbose...
+              Use verbose output
+          --color <COLOR_CHOICE>
+              Control colors in output [default: auto] [possible values: auto, always, never]
+          --native-tls
+              Whether to load TLS certificates from the platform's native certificate store [env:
+              UV_NATIVE_TLS=]
+          --offline
+              Disable network access
+          --allow-insecure-host <ALLOW_INSECURE_HOST>
+              Allow insecure connections to a host [env: UV_INSECURE_HOST=]
+          --no-progress
+              Hide all progress outputs [env: UV_NO_PROGRESS=]
+          --directory <DIRECTORY>
+              Change to the given directory prior to running the command
+          --project <PROJECT>
+              Run the command within the given project directory
+          --config-file <CONFIG_FILE>
+              The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
+          --no-config
+              Avoid discovering configuration files (`pyproject.toml`, `uv.toml`) [env: UV_NO_CONFIG=]
+      -h, --help
+              Display the concise help for this command
+      -V, --version
+              Display the uv version
 
     Use `uv help <command>` for more information on a specific command.
 
@@ -116,22 +127,33 @@ fn help_flag() {
               Disable automatic downloads of Python. [env: "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet                      Do not print any output
-      -v, --verbose...                 Use verbose output
-          --color <COLOR_CHOICE>       Control colors in output [default: auto] [possible values: auto,
-                                       always, never]
-          --native-tls                 Whether to load TLS certificates from the platform's native
-                                       certificate store [env: UV_NATIVE_TLS=]
-          --offline                    Disable network access
-          --no-progress                Hide all progress outputs [env: UV_NO_PROGRESS=]
-          --directory <DIRECTORY>      Change to the given directory prior to running the command
-          --project <PROJECT>          Run the command within the given project directory
-          --config-file <CONFIG_FILE>  The path to a `uv.toml` file to use for configuration [env:
-                                       UV_CONFIG_FILE=]
-          --no-config                  Avoid discovering configuration files (`pyproject.toml`,
-                                       `uv.toml`) [env: UV_NO_CONFIG=]
-      -h, --help                       Display the concise help for this command
-      -V, --version                    Display the uv version
+      -q, --quiet
+              Do not print any output
+      -v, --verbose...
+              Use verbose output
+          --color <COLOR_CHOICE>
+              Control colors in output [default: auto] [possible values: auto, always, never]
+          --native-tls
+              Whether to load TLS certificates from the platform's native certificate store [env:
+              UV_NATIVE_TLS=]
+          --offline
+              Disable network access
+          --allow-insecure-host <ALLOW_INSECURE_HOST>
+              Allow insecure connections to a host [env: UV_INSECURE_HOST=]
+          --no-progress
+              Hide all progress outputs [env: UV_NO_PROGRESS=]
+          --directory <DIRECTORY>
+              Change to the given directory prior to running the command
+          --project <PROJECT>
+              Run the command within the given project directory
+          --config-file <CONFIG_FILE>
+              The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
+          --no-config
+              Avoid discovering configuration files (`pyproject.toml`, `uv.toml`) [env: UV_NO_CONFIG=]
+      -h, --help
+              Display the concise help for this command
+      -V, --version
+              Display the uv version
 
     Use `uv help` for more details.
 
@@ -184,22 +206,33 @@ fn help_short_flag() {
               Disable automatic downloads of Python. [env: "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet                      Do not print any output
-      -v, --verbose...                 Use verbose output
-          --color <COLOR_CHOICE>       Control colors in output [default: auto] [possible values: auto,
-                                       always, never]
-          --native-tls                 Whether to load TLS certificates from the platform's native
-                                       certificate store [env: UV_NATIVE_TLS=]
-          --offline                    Disable network access
-          --no-progress                Hide all progress outputs [env: UV_NO_PROGRESS=]
-          --directory <DIRECTORY>      Change to the given directory prior to running the command
-          --project <PROJECT>          Run the command within the given project directory
-          --config-file <CONFIG_FILE>  The path to a `uv.toml` file to use for configuration [env:
-                                       UV_CONFIG_FILE=]
-          --no-config                  Avoid discovering configuration files (`pyproject.toml`,
-                                       `uv.toml`) [env: UV_NO_CONFIG=]
-      -h, --help                       Display the concise help for this command
-      -V, --version                    Display the uv version
+      -q, --quiet
+              Do not print any output
+      -v, --verbose...
+              Use verbose output
+          --color <COLOR_CHOICE>
+              Control colors in output [default: auto] [possible values: auto, always, never]
+          --native-tls
+              Whether to load TLS certificates from the platform's native certificate store [env:
+              UV_NATIVE_TLS=]
+          --offline
+              Disable network access
+          --allow-insecure-host <ALLOW_INSECURE_HOST>
+              Allow insecure connections to a host [env: UV_INSECURE_HOST=]
+          --no-progress
+              Hide all progress outputs [env: UV_NO_PROGRESS=]
+          --directory <DIRECTORY>
+              Change to the given directory prior to running the command
+          --project <PROJECT>
+              Run the command within the given project directory
+          --config-file <CONFIG_FILE>
+              The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
+          --no-config
+              Avoid discovering configuration files (`pyproject.toml`, `uv.toml`) [env: UV_NO_CONFIG=]
+      -h, --help
+              Display the concise help for this command
+      -V, --version
+              Display the uv version
 
     Use `uv help` for more details.
 
@@ -223,7 +256,7 @@ fn help_subcommand() {
     interpreters are found by searching for Python executables in the `PATH` environment
     variable.
 
-    On Windows, the `py` launcher is also invoked to find Python executables.
+    On Windows, the registry is also searched for Python executables.
 
     By default, uv will download Python if a version cannot be found. This behavior can be
     disabled with the `--no-python-downloads` flag or the `python-downloads` setting.
@@ -342,6 +375,20 @@ fn help_subcommand() {
               
               When disabled, uv will only use locally cached data and locally available files.
 
+          --allow-insecure-host <ALLOW_INSECURE_HOST>
+              Allow insecure connections to a host.
+              
+              Can be provided multiple times.
+              
+              Expects to receive either a hostname (e.g., `localhost`), a host-port pair (e.g.,
+              `localhost:8080`), or a URL (e.g., `https://localhost`).
+              
+              WARNING: Hosts included in this list will not be verified against the system's certificate
+              store. Only use `--allow-insecure-host` in a secure network with verified sources, as it
+              bypasses SSL verification and could expose you to MITM attacks.
+              
+              [env: UV_INSECURE_HOST=]
+
           --no-progress
               Hide all progress outputs.
               
@@ -525,6 +572,20 @@ fn help_subsubcommand() {
               
               When disabled, uv will only use locally cached data and locally available files.
 
+          --allow-insecure-host <ALLOW_INSECURE_HOST>
+              Allow insecure connections to a host.
+              
+              Can be provided multiple times.
+              
+              Expects to receive either a hostname (e.g., `localhost`), a host-port pair (e.g.,
+              `localhost:8080`), or a URL (e.g., `https://localhost`).
+              
+              WARNING: Hosts included in this list will not be verified against the system's certificate
+              store. Only use `--allow-insecure-host` in a secure network with verified sources, as it
+              bypasses SSL verification and could expose you to MITM attacks.
+              
+              [env: UV_INSECURE_HOST=]
+
           --no-progress
               Hide all progress outputs.
               
@@ -613,22 +674,33 @@ fn help_flag_subcommand() {
               Disable automatic downloads of Python. [env: "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet                      Do not print any output
-      -v, --verbose...                 Use verbose output
-          --color <COLOR_CHOICE>       Control colors in output [default: auto] [possible values: auto,
-                                       always, never]
-          --native-tls                 Whether to load TLS certificates from the platform's native
-                                       certificate store [env: UV_NATIVE_TLS=]
-          --offline                    Disable network access
-          --no-progress                Hide all progress outputs [env: UV_NO_PROGRESS=]
-          --directory <DIRECTORY>      Change to the given directory prior to running the command
-          --project <PROJECT>          Run the command within the given project directory
-          --config-file <CONFIG_FILE>  The path to a `uv.toml` file to use for configuration [env:
-                                       UV_CONFIG_FILE=]
-          --no-config                  Avoid discovering configuration files (`pyproject.toml`,
-                                       `uv.toml`) [env: UV_NO_CONFIG=]
-      -h, --help                       Display the concise help for this command
-      -V, --version                    Display the uv version
+      -q, --quiet
+              Do not print any output
+      -v, --verbose...
+              Use verbose output
+          --color <COLOR_CHOICE>
+              Control colors in output [default: auto] [possible values: auto, always, never]
+          --native-tls
+              Whether to load TLS certificates from the platform's native certificate store [env:
+              UV_NATIVE_TLS=]
+          --offline
+              Disable network access
+          --allow-insecure-host <ALLOW_INSECURE_HOST>
+              Allow insecure connections to a host [env: UV_INSECURE_HOST=]
+          --no-progress
+              Hide all progress outputs [env: UV_NO_PROGRESS=]
+          --directory <DIRECTORY>
+              Change to the given directory prior to running the command
+          --project <PROJECT>
+              Run the command within the given project directory
+          --config-file <CONFIG_FILE>
+              The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
+          --no-config
+              Avoid discovering configuration files (`pyproject.toml`, `uv.toml`) [env: UV_NO_CONFIG=]
+      -h, --help
+              Display the concise help for this command
+      -V, --version
+              Display the uv version
 
     Use `uv help python` for more details.
 
@@ -668,22 +740,33 @@ fn help_flag_subsubcommand() {
               Disable automatic downloads of Python. [env: "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet                      Do not print any output
-      -v, --verbose...                 Use verbose output
-          --color <COLOR_CHOICE>       Control colors in output [default: auto] [possible values: auto,
-                                       always, never]
-          --native-tls                 Whether to load TLS certificates from the platform's native
-                                       certificate store [env: UV_NATIVE_TLS=]
-          --offline                    Disable network access
-          --no-progress                Hide all progress outputs [env: UV_NO_PROGRESS=]
-          --directory <DIRECTORY>      Change to the given directory prior to running the command
-          --project <PROJECT>          Run the command within the given project directory
-          --config-file <CONFIG_FILE>  The path to a `uv.toml` file to use for configuration [env:
-                                       UV_CONFIG_FILE=]
-          --no-config                  Avoid discovering configuration files (`pyproject.toml`,
-                                       `uv.toml`) [env: UV_NO_CONFIG=]
-      -h, --help                       Display the concise help for this command
-      -V, --version                    Display the uv version
+      -q, --quiet
+              Do not print any output
+      -v, --verbose...
+              Use verbose output
+          --color <COLOR_CHOICE>
+              Control colors in output [default: auto] [possible values: auto, always, never]
+          --native-tls
+              Whether to load TLS certificates from the platform's native certificate store [env:
+              UV_NATIVE_TLS=]
+          --offline
+              Disable network access
+          --allow-insecure-host <ALLOW_INSECURE_HOST>
+              Allow insecure connections to a host [env: UV_INSECURE_HOST=]
+          --no-progress
+              Hide all progress outputs [env: UV_NO_PROGRESS=]
+          --directory <DIRECTORY>
+              Change to the given directory prior to running the command
+          --project <PROJECT>
+              Run the command within the given project directory
+          --config-file <CONFIG_FILE>
+              The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
+          --no-config
+              Avoid discovering configuration files (`pyproject.toml`, `uv.toml`) [env: UV_NO_CONFIG=]
+      -h, --help
+              Display the concise help for this command
+      -V, --version
+              Display the uv version
 
     ----- stderr -----
     "###);
@@ -814,22 +897,33 @@ fn help_with_global_option() {
               Disable automatic downloads of Python. [env: "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet                      Do not print any output
-      -v, --verbose...                 Use verbose output
-          --color <COLOR_CHOICE>       Control colors in output [default: auto] [possible values: auto,
-                                       always, never]
-          --native-tls                 Whether to load TLS certificates from the platform's native
-                                       certificate store [env: UV_NATIVE_TLS=]
-          --offline                    Disable network access
-          --no-progress                Hide all progress outputs [env: UV_NO_PROGRESS=]
-          --directory <DIRECTORY>      Change to the given directory prior to running the command
-          --project <PROJECT>          Run the command within the given project directory
-          --config-file <CONFIG_FILE>  The path to a `uv.toml` file to use for configuration [env:
-                                       UV_CONFIG_FILE=]
-          --no-config                  Avoid discovering configuration files (`pyproject.toml`,
-                                       `uv.toml`) [env: UV_NO_CONFIG=]
-      -h, --help                       Display the concise help for this command
-      -V, --version                    Display the uv version
+      -q, --quiet
+              Do not print any output
+      -v, --verbose...
+              Use verbose output
+          --color <COLOR_CHOICE>
+              Control colors in output [default: auto] [possible values: auto, always, never]
+          --native-tls
+              Whether to load TLS certificates from the platform's native certificate store [env:
+              UV_NATIVE_TLS=]
+          --offline
+              Disable network access
+          --allow-insecure-host <ALLOW_INSECURE_HOST>
+              Allow insecure connections to a host [env: UV_INSECURE_HOST=]
+          --no-progress
+              Hide all progress outputs [env: UV_NO_PROGRESS=]
+          --directory <DIRECTORY>
+              Change to the given directory prior to running the command
+          --project <PROJECT>
+              Run the command within the given project directory
+          --config-file <CONFIG_FILE>
+              The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
+          --no-config
+              Avoid discovering configuration files (`pyproject.toml`, `uv.toml`) [env: UV_NO_CONFIG=]
+      -h, --help
+              Display the concise help for this command
+      -V, --version
+              Display the uv version
 
     Use `uv help <command>` for more information on a specific command.
 
@@ -919,22 +1013,33 @@ fn help_with_no_pager() {
               Disable automatic downloads of Python. [env: "UV_PYTHON_DOWNLOADS=never"]
 
     Global options:
-      -q, --quiet                      Do not print any output
-      -v, --verbose...                 Use verbose output
-          --color <COLOR_CHOICE>       Control colors in output [default: auto] [possible values: auto,
-                                       always, never]
-          --native-tls                 Whether to load TLS certificates from the platform's native
-                                       certificate store [env: UV_NATIVE_TLS=]
-          --offline                    Disable network access
-          --no-progress                Hide all progress outputs [env: UV_NO_PROGRESS=]
-          --directory <DIRECTORY>      Change to the given directory prior to running the command
-          --project <PROJECT>          Run the command within the given project directory
-          --config-file <CONFIG_FILE>  The path to a `uv.toml` file to use for configuration [env:
-                                       UV_CONFIG_FILE=]
-          --no-config                  Avoid discovering configuration files (`pyproject.toml`,
-                                       `uv.toml`) [env: UV_NO_CONFIG=]
-      -h, --help                       Display the concise help for this command
-      -V, --version                    Display the uv version
+      -q, --quiet
+              Do not print any output
+      -v, --verbose...
+              Use verbose output
+          --color <COLOR_CHOICE>
+              Control colors in output [default: auto] [possible values: auto, always, never]
+          --native-tls
+              Whether to load TLS certificates from the platform's native certificate store [env:
+              UV_NATIVE_TLS=]
+          --offline
+              Disable network access
+          --allow-insecure-host <ALLOW_INSECURE_HOST>
+              Allow insecure connections to a host [env: UV_INSECURE_HOST=]
+          --no-progress
+              Hide all progress outputs [env: UV_NO_PROGRESS=]
+          --directory <DIRECTORY>
+              Change to the given directory prior to running the command
+          --project <PROJECT>
+              Run the command within the given project directory
+          --config-file <CONFIG_FILE>
+              The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
+          --no-config
+              Avoid discovering configuration files (`pyproject.toml`, `uv.toml`) [env: UV_NO_CONFIG=]
+      -h, --help
+              Display the concise help for this command
+      -V, --version
+              Display the uv version
 
     Use `uv help <command>` for more information on a specific command.
 
