@@ -50,6 +50,28 @@ fn create_venv() {
     context.venv.assert(predicates::path::is_dir());
 }
 
+#[test]
+fn create_venv_313() {
+    let context = TestContext::new_with_versions(&["3.13"]);
+
+    uv_snapshot!(context.filters(), context.venv()
+        .arg(context.venv.as_os_str())
+        .arg("--python")
+        .arg("3.13"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Using CPython 3.13.[X] interpreter at: [PYTHON-3.13]
+    Creating virtual environment at: .venv
+    Activate with: source .venv/[BIN]/activate
+    "###
+    );
+
+    context.venv.assert(predicates::path::is_dir());
+}
+
 #[test]
 fn create_venv_project_environment() -> Result<()> {
     let context = TestContext::new_with_versions(&["3.12"]);
@@ -453,6 +475,20 @@ fn create_venv_respects_pyproject_requires_python() -> Result<()> {
 
     context.venv.assert(predicates::path::is_dir());
 
+    // We warn if we receive an incompatible version
+    uv_snapshot!(context.filters(), context.venv().arg("--python").arg("3.11"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Using CPython 3.11.[X] interpreter at: [PYTHON-3.11]
+    warning: The requested interpreter resolved to Python 3.11.[X], which is incompatible with the project's Python requirement: `>=3.12`
+    Creating virtual environment at: .venv
+    Activate with: source .venv/[BIN]/activate
+    "###
+    );
+
     Ok(())
 }
 
@@ -605,7 +641,7 @@ fn create_venv_unknown_python_minor() {
         ----- stdout -----
 
         ----- stderr -----
-          × No interpreter found for Python 3.100 in managed installations, system path, or `py` launcher
+          × No interpreter found for Python 3.100 in managed installations, search path, or registry
         "###
         );
     } else {
@@ -615,7 +651,7 @@ fn create_venv_unknown_python_minor() {
         ----- stdout -----
 
         ----- stderr -----
-          × No interpreter found for Python 3.100 in managed installations or system path
+          × No interpreter found for Python 3.100 in managed installations or search path
         "###
         );
     }
@@ -643,7 +679,7 @@ fn create_venv_unknown_python_patch() {
         ----- stdout -----
 
         ----- stderr -----
-          × No interpreter found for Python 3.12.100 in managed installations, system path, or `py` launcher
+          × No interpreter found for Python 3.12.100 in managed installations, search path, or registry
         "###
         );
     } else {
@@ -653,7 +689,7 @@ fn create_venv_unknown_python_patch() {
         ----- stdout -----
 
         ----- stderr -----
-          × No interpreter found for Python 3.12.100 in managed installations or system path
+          × No interpreter found for Python 3.12.100 in managed installations or search path
         "###
         );
     }
